@@ -133,7 +133,7 @@ class BaseWritter(Writer):
 
     def gcc_build_setup(self, name, compiler, compiler_var,
                         opts: BuildOptions):
-        global_build_opts: BuildOptions = self.compiler_build_opts.get(
+        global_build_opts = self.compiler_build_opts.get(
             compiler, BuildOptions())
         opts = opts.merge(global_build_opts)
         includes = " ".join(["-I" + str(i) for i in opts.includes])
@@ -144,15 +144,15 @@ class BaseWritter(Writer):
         rule_name = name + "_cxx"
         self.rule(
             rule_name,
-            f"${compiler_var} -MMD -MT $out -MF $out.d {includes} {cflags} -c $in -o $out {post_cflags}",
-            description=f"compile $out",
+            "${} -MMD -MT $out -MF $out.d {} {} -c $in -o $out {}".format(compiler_var, includes, cflags, post_cflags),
+            description="compile $out",
             depfile="$out.d",
             deps="gcc")
         self.newline()
         return rule_name
 
     def gcc_link_setup(self, name, linker, linker_name, opts: LinkOptions):
-        global_build_opts: LinkOptions = self.compiler_link_opts.get(
+        global_build_opts = self.compiler_link_opts.get(
             linker, LinkOptions())
         opts = opts.merge(global_build_opts)
         ldflags = " ".join(opts.ldflags)
@@ -161,14 +161,14 @@ class BaseWritter(Writer):
         rule_name = name + "_ld"
         self.rule(
             rule_name,
-            f"${linker_name} $in {libs_str} {libpaths_str} {ldflags} -o $out",
-            description=f"link $out")
+            "${} $in {} {} {} -o $out".format(linker_name, libs_str, libpaths_str, ldflags),
+            description="link $out")
         self.newline()
         return rule_name
 
     def msvc_build_setup(self, name, compiler, compiler_var,
                          opts: BuildOptions):
-        global_build_opts: BuildOptions = self.compiler_build_opts.get(
+        global_build_opts = self.compiler_build_opts.get(
             compiler, BuildOptions())
         opts = opts.merge(global_build_opts)
         includes = " ".join(
@@ -180,13 +180,13 @@ class BaseWritter(Writer):
         rule_name = name + "_cxx"
         self.rule(
             rule_name,
-            f"${compiler_var} {includes} {cflags} -c $in /Fo$out ${post_cflags}"
+            "${} {} {} -c $in /Fo$out ${}".format(compiler_var, includes, cflags, post_cflags)
         )
         self.newline()
         return rule_name
 
     def msvc_link_setup(self, name, linker, linker_name, opts: LinkOptions):
-        global_build_opts: LinkOptions = self.compiler_link_opts.get(
+        global_build_opts = self.compiler_link_opts.get(
             linker, LinkOptions())
         opts = opts.merge(global_build_opts)
         ldflags = " ".join(opts.ldflags)
@@ -196,14 +196,14 @@ class BaseWritter(Writer):
         rule_name = name + "_ld"
         self.rule(
             rule_name,
-            f"${linker_name} /link /nologo $in {libs_str} {libpaths_str} {ldflags} /out:$out ",
-            description=f"link msvc $out")
+            "${} /link /nologo $in {} {} {} /out:$out".format(linker_name, libs_str, libpaths_str, ldflags),
+            description="link msvc $out")
         self.newline()
         return rule_name
 
     def nvcc_build_setup(self, name, compiler, compiler_var,
                          opts: BuildOptions):
-        global_build_opts: BuildOptions = self.compiler_build_opts.get(
+        global_build_opts = self.compiler_build_opts.get(
             compiler, BuildOptions())
         opts = opts.merge(global_build_opts)
         includes = " ".join(["-I" + str(i) for i in opts.includes])
@@ -214,13 +214,13 @@ class BaseWritter(Writer):
         rule_name = name + "_cuda"
         self.rule(
             rule_name,
-            f"${compiler_var} {includes} {cflags} -c $in -o $out {post_cflags}",
-            description=f"nvcc cxx $out")
+            "${} {} {} -c $in -o $out {}".format(compiler_var, includes, cflags, post_cflags),
+            description="nvcc cxx $out")
         self.newline()
         return rule_name
 
     def nvcc_link_setup(self, name, linker, linker_name, opts: LinkOptions):
-        global_build_opts: LinkOptions = self.compiler_link_opts.get(
+        global_build_opts = self.compiler_link_opts.get(
             linker, LinkOptions())
         opts = opts.merge(global_build_opts)
         ldflags = " ".join(opts.ldflags)
@@ -229,25 +229,25 @@ class BaseWritter(Writer):
         rule_name = name + "_ld"
         self.rule(
             rule_name,
-            f"${linker_name} $in {libs_str} {libpaths_str} {ldflags} -o $out",
-            description=f"link $out")
+            "${} $in {} {} {} -o $out".format(linker_name, libs_str, libpaths_str, ldflags),
+            description="link $out")
         self.newline()
         return rule_name
 
     def create_linker_rule(self, linker, target_name, link_opts: LinkOptions):
-        link_name = f"{linker}_{target_name}"
+        link_name = "{}_{}".format(linker, target_name)
         linker_path = None
         if linker in self.linker_to_path:
             linker_path = self.linker_to_path[linker]
         if linker == "g++":
             # ++ can't be used in name
-            link_name = f"gplusplus_{target_name}"
+            link_name = "gplusplus_{}".format(target_name)
             self.variable(link_name,
                           'g++' if linker_path is None else linker_path)
             return self.gcc_link_setup(target_name, linker, link_name,
                                        link_opts)
         elif linker == "clang++":
-            link_name = f"clang_{target_name}"
+            link_name = "clang_{}".format(target_name)
             self.variable(link_name,
                           "clang++" if linker_path is None else linker_path)
             return self.gcc_link_setup(target_name, linker, link_name,
@@ -449,7 +449,7 @@ def group_dict_by_split(data: Dict[str, List[Any]], split: str = ","):
     """convert {gcc,clang++: [xxx], clang++: [yyy]}
     to {gcc: [xxx], clang++: [xxx, yyy]}
     """
-    new_data: Dict[str, List[Any]] = {}
+    new_data = {} # type: Dict[str, List[Any]]
     for k, v in data.items():
         ks = k.split(split)
         for k_ in ks:
@@ -506,7 +506,6 @@ def create_simple_ninja(target,
                       target_filename, shared)
     return writer.content, target_filename
 
-
 def build_simple_ninja(target,
                        build_dir,
                        sources,
@@ -532,8 +531,31 @@ def build_simple_ninja(target,
     cmds = ["ninja"]
     if verbose:
         cmds.append("-v")
-    subprocess.check_call(cmds, cwd=str(build_dir))
-    return target_filename
+    if compat.Python3_7AndLater:
+        proc = subprocess.Popen(cmds, cwd=str(build_dir),
+                                stdout=subprocess.PIPE,
+                                text=True)
+    else:
+        proc = subprocess.Popen(cmds, cwd=str(build_dir),
+                                stdout=subprocess.PIPE,
+                                universal_newlines=True)
+    output = ''
+    while True:
+        chunk_or_line = proc.stdout.readline()
+        if not chunk_or_line:
+            break
+        output += chunk_or_line
+        # print(chunk_or_line.encode("utf-8"))
+        if not "ninja: no work to do" in chunk_or_line:
+            print(chunk_or_line, end='')
+    proc.wait()
+    if proc.returncode:
+        raise subprocess.CalledProcessError(proc.returncode, cmds)
+
+    no_work_to_do = False
+    if "ninja: no work to do" in output:
+        no_work_to_do = True
+    return target_filename, no_work_to_do
 
 
 def run_simple_ninja(target,
